# -*- coding: utf-8 -*-
r"""
Matrix cocyles

EXAMPLES:

The 1-cylinders of ARP transformation given as matrices::

    sage: from slabbe.matrix_cocycle import cocycles
    sage: ARP = cocycles.ARP()
    sage: list(zip(*ARP.n_cylinders_iterator(1)))
    [(word: 1,
      word: 2,
      word: 3,
      word: 123,
      word: 132,
      word: 213,
      word: 231,
      word: 312,
      word: 321),
     (
    [1 1 1]  [1 0 0]  [1 0 0]  [1 0 1]  [1 1 0]  [1 1 1]  [2 1 1]  [1 1 1]
    [0 1 0]  [1 1 1]  [0 1 0]  [1 1 1]  [1 2 1]  [0 1 1]  [1 1 0]  [1 2 1]
    [0 0 1], [0 0 1], [1 1 1], [1 1 2], [1 1 1], [1 1 2], [1 1 1], [0 1 1],
    <BLANKLINE>
    [2 1 1]
    [1 1 1]
    [1 0 1]
    )]

Ces calculs illustrent le bounded distorsion de ratio=4 pour ARP
multiplicatif (2 avril 2014)::

    sage: T = cocycles.Sorted_ARPMulti(2)
    sage: T.distorsion_max(1, p=oo)
    5
    sage: T.distorsion_max(2, p=oo)
    7
    sage: T.distorsion_max(3, p=oo)
    22/3
    sage: T.distorsion_max(4, p=oo)    # long time (4s)
    62/17

::

    sage: T = cocycles.Sorted_ARPMulti(3)
    sage: T.distorsion_max(1, p=oo)
    7
    sage: T.distorsion_max(2, p=oo)
    9
    sage: T.distorsion_max(3, p=oo)
    19/2
    sage: T.distorsion_max(4, p=oo)  # long time (47s)
    161/43

.. TODO::

    - Fix the semi_norm_v bug using linear programming.

"""
#*****************************************************************************
#       Copyright (C) 2014-2015 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
import itertools
from collections import Counter
from sage.matrix.constructor import matrix
from sage.misc.cachefunc import cached_method
from sage.misc.misc_c import prod
from slabbe.language import Language, FiniteLanguage

######################
# Matrix Cocycle
######################
class MatrixCocycle(object):
    r"""
    Matrix cocycle

    INPUT:

    - ``gens`` -- list, tuple or dict; the matrices. Keys 0,...,n-1 are
      used for list and tuple.
    - ``cone`` -- dict or matrix or None (default: None); the cone for each
      matrix generators.  If it is a matrix, then it serves as the cone for
      all matrices. The cone is defined by the columns of the matrix. If
      None, then the cone is the identity matrix.
    - ``language`` -- regular language or None (default: None); if None,
      the language is the full shift.

    EXAMPLES::

        sage: from slabbe.matrix_cocycle import MatrixCocycle
        sage: B1 = matrix(3, [1,0,0, 0,1,0, 0,1,1])
        sage: B2 = matrix(3, [1,0,0, 0,0,1, 0,1,1])
        sage: B3 = matrix(3, [0,1,0, 0,0,1, 1,0,1])
        sage: gens = {'1':B1, '2':B2, '3':B3}
        sage: cone = matrix(3, [1,1,1,0,1,1,0,0,1])
        sage: MatrixCocycle(gens, cone)
        Cocycle with 3 gens over Language of finite words over alphabet ['1', '2', '3']
    """
    def __init__(self, gens, cone=None, language=None):
        r"""
        EXAMPLES::

            sage: from slabbe.matrix_cocycle import MatrixCocycle
            sage: gens = {'A':matrix(3, [1,0,0, 0,1,0, 0,1,1])}
            sage: cone = identity_matrix(3)
            sage: MatrixCocycle(gens, cone)
            Cocycle with 1 gens over Language of finite words over alphabet ['A']
        """
        if isinstance(gens, dict):
            self._gens = gens
        elif isinstance(gens, (list, tuple)):
            self._gens = dict(enumerate(gens))
        else:
            raise ValueError("gens must be a list, tuple or a dict")
        if cone is None:
            ID = self.identity_matrix()
            self._cone_dict = {letter:ID for letter in self._gens.keys()}
        elif isinstance(cone, dict):
            self._cone_dict = cone
        else:
            self._cone_dict = {letter:cone for letter in self._gens.keys()}
        if language is None:
            self._language = Language(sorted(self._gens.keys()))
        else:
            self._language = language

    def __repr__(self):
        r"""
        EXAMPLES::

            sage: from slabbe.matrix_cocycle import MatrixCocycle
            sage: gens = {'A':matrix(3, [1,0,0, 0,1,0, 0,1,1])}
            sage: cone = identity_matrix(3)
            sage: MatrixCocycle(gens, cone)
            Cocycle with 1 gens over Language of finite words over alphabet ['A']
        """
        s = "Cocycle with {} gens over {}"
        return s.format(len(self._gens), self._language)

    def gens(self):
        return self._gens
    def gens_inverses(self):
        r"""
        Return a dictionary of the inverses of the generators.

        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: coc = cocycles.Brun()
            sage: sorted(coc.gens_inverses().keys())
            [123, 132, 213, 231, 312, 321]
            sage: sorted(coc.gens_inverses().values())
            [
            [ 1 -1  0]  [ 1  0 -1]  [ 1  0  0]  [ 1  0  0]  [ 1  0  0]  [ 1  0  0]
            [ 0  1  0]  [ 0  1  0]  [-1  1  0]  [ 0  1 -1]  [ 0  1  0]  [ 0  1  0]
            [ 0  0  1], [ 0  0  1], [ 0  0  1], [ 0  0  1], [-1  0  1], [ 0 -1  1]
            ]

        If possible, the ring is the Integer ring::

            sage: coc = cocycles.Reverse()
            sage: sorted(coc.gens_inverses().values())
            [
            [-1/2  1/2  1/2]  [ 1 -1 -1]  [ 1  0  0]  [ 1  0  0]
            [ 1/2 -1/2  1/2]  [ 0  1  0]  [-1  1 -1]  [ 0  1  0]
            [ 1/2  1/2 -1/2], [ 0  0  1], [ 0  0  1], [-1 -1  1]
            ]
            sage: [m.parent() for m in _]
            [Full MatrixSpace of 3 by 3 dense matrices over Rational Field,
             Full MatrixSpace of 3 by 3 dense matrices over Integer Ring,
             Full MatrixSpace of 3 by 3 dense matrices over Integer Ring,
             Full MatrixSpace of 3 by 3 dense matrices over Integer Ring]
        """
        from sage.rings.integer_ring import ZZ
        D = {}
        for k,v in self.gens().items():
            M = v.inverse()
            try:
                M_ZZ = M.change_ring(ZZ)
            except TypeError:
                pass
            else:
                M = M_ZZ
            D[k] = M
        return D

    def cone_dict(self):
        return self._cone_dict
    def cone(self, key):
        return self._cone_dict[key]
    def language(self):
        return self._language

    @cached_method
    def identity_matrix(self):
        values = list(self._gens.values())
        return values[0].parent().one()

    def word_to_matrix(self, w):
        r"""
        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: C = cocycles.Sorted_ARP()
            sage: C.word_to_matrix(Word())
            [1 0 0]
            [0 1 0]
            [0 0 1]
        """
        return prod((self._gens[a] for a in w), z=self.identity_matrix())

    def n_words_iterator(self, n):
        r"""
        EXAMPLES::
            
            sage: from slabbe.matrix_cocycle import cocycles
            sage: ARP = cocycles.Sorted_ARP()
            sage: list(ARP.n_words_iterator(1))
            [word: A1, word: A2, word: A3, word: P1, word: P2, word: P3]
        """
        return self._language.words_of_length_iterator(n)

    def n_matrices_iterator(self, n):
        r"""
        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: ARP = cocycles.Sorted_ARP()
            sage: A,B = zip(*list(ARP.n_matrices_iterator(1)))
            sage: A
            (word: A1, word: A2, word: A3, word: P1, word: P2, word: P3)
            sage: B
            (
            [1 0 0]  [1 0 0]  [0 1 0]  [0 1 0]  [0 0 1]  [0 0 1]
            [0 1 0]  [0 0 1]  [0 0 1]  [0 1 1]  [1 0 1]  [0 1 1]
            [1 1 1], [1 1 1], [1 1 1], [1 1 1], [1 1 1], [1 1 1]
            )
        """
        for w in self.n_words_iterator(n):
            yield w, self.word_to_matrix(w)

    def n_matrices_eigenvalues_iterator(self,n):
        r"""
        Return the eigenvalues of the matrices of level n.

        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: ARP = cocycles.ARP()
            sage: list(ARP.n_matrices_eigenvalues_iterator(1))
            [(word: 1, [1, 1, 1]),
             (word: 2, [1, 1, 1]),
             (word: 3, [1, 1, 1]),
             (word: 123, [1, 1, 1]),
             (word: 132, [1, 1, 1]),
             (word: 213, [1, 1, 1]),
             (word: 231, [1, 1, 1]),
             (word: 312, [1, 1, 1]),
             (word: 321, [1, 1, 1])]

        ::

            sage: B = cocycles.Sorted_Brun()
            sage: list(B.n_matrices_eigenvalues_iterator(1))
            [(word: 1, [1, 1, 1]),
             (word: 2, [1, -0.618033988749895?, 1.618033988749895?]),
             (word: 3, [1.465571231876768?, 
                        -0.2327856159383841? - 0.7925519925154479?*I,
                        -0.2327856159383841? + 0.7925519925154479?*I])]
        """
        for w,m in self.n_matrices_iterator(n):
            yield w, m.eigenvalues()

    def n_matrices_pinching_iterator(self,n):
        r"""
        Return the pinching matrices of level n.

        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: ARP = cocycles.ARP()
            sage: list(ARP.n_matrices_pinching_iterator(0))
            []
            sage: list(ARP.n_matrices_pinching_iterator(1))
            []
            sage: list(ARP.n_matrices_pinching_iterator(2))
            []
            sage: L = list(ARP.n_matrices_pinching_iterator(3))
            sage: L[0]
            (
                           [4 5 2]
                           [2 3 1]
            word: 1,2,213, [1 1 1]
            )
        """
        for w,m in self.n_matrices_iterator(n):
            p = m.charpoly()
            d = p.discriminant()
            if p.is_irreducible() and d > 0 and not d.is_square():
                yield w, m

    def n_matrices_eigenvectors(self,n, verbose=False):
        r"""
        Return the left and right eigenvectors of the matrices of level n.

        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: C = cocycles.ARP()
            sage: C.n_matrices_eigenvectors(1)
            [(word: 1, (1.0, 0.0, 0.0), (0.0, 0.0, 1.0)),
             (word: 2, (0.0, 1.0, 0.0), (1.0, 0.0, 0.0)),
             (word: 3, (0.0, 0.0, 1.0), (1.0, 0.0, 0.0)),
             (word: 123, (0.0, 0.0, 1.0), (1.0, 0.0, 0.0)),
             (word: 132, (0.0, 1.0, 0.0), (1.0, 0.0, 0.0)),
             (word: 213, (0.0, 0.0, 1.0), (0.0, 1.0, 0.0)),
             (word: 231, (1.0, 0.0, 0.0), (0.0, 1.0, 0.0)),
             (word: 312, (0.0, 1.0, 0.0), (0.0, 0.0, 1.0)),
             (word: 321, (1.0, 0.0, 0.0), (0.0, 0.0, 1.0))]
        """
        R = []
        for w,m in self.n_matrices_iterator(n):
            try:
                a,v_right = perron_right_eigenvector(m)
                b,v_left = perron_right_eigenvector(m.transpose())
            except ValueError:
                print("problem with :\n",m)
            else:
                R.append((w, v_right,v_left))
                if verbose:
                    print("indices of matrices:", w)
                    print(m)
                    print("eigenvectors:", v_right, v_left)
        return R
    @cached_method
    def n_matrices_non_pisot(self,n, verbose=False):
        r"""
        Return the list of non pisot matrices (as list of indices of base
        matrices).

        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: ARP = cocycles.Sorted_ARP()
            sage: ARP.n_matrices_non_pisot(1)
            [word: A1, word: A2]
            sage: ARP.n_matrices_non_pisot(2)   # long time (1s)
            [word: A1,A1, word: A1,A2, word: A2,A1, word: A2,A2]
            sage: ARP.n_matrices_non_pisot(3)   # long time (11s)
            [word: A1,A1,A1,
             word: A1,A1,A2,
             word: A1,A2,A1,
             word: A1,A2,A2,
             word: A2,A1,A1,
             word: A2,A1,A2,
             word: A2,A2,A1,
             word: A2,A2,A2]
            sage: len(ARP.n_matrices_non_pisot(4))  # long time
            16

        ::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: B = cocycles.Sorted_Brun()
            sage: B.n_matrices_non_pisot(2)
            [word: 11, word: 12, word: 21, word: 22]
            sage: B.n_matrices_non_pisot(3)
            [word: 111,
             word: 112,
             word: 121,
             word: 122,
             word: 211,
             word: 212,
             word: 221,
             word: 222]

        """
        return [w for w in self.n_words_iterator(n) if not self.is_pisot(w)]

    def n_matrices_semi_norm_iterator(self, n, p=2):
        r"""
        EXAMPLES:

        For the 1-norm, all matrices contracts the hyperplane::
            
            sage: from slabbe.matrix_cocycle import cocycles
            sage: C = cocycles.ARP()
            sage: it = C.n_matrices_semi_norm_iterator(1, p=1)
            sage: for _ in range(5): print(next(it)) # tolerance 0.0001
            (word: 1, 1.0, False)
            (word: 2, 1.0, False)
            (word: 3, 1.0, False)
            (word: 123, 0.9999885582839877, False)
            (word: 132, 0.9999854006354785, False)

        For the 2-norm, AR matrices do not contract::

            sage: it = C.n_matrices_semi_norm_iterator(1, p=2)
            sage: for w,s,b in it: print(w,s,b)  # long time (6s)
            A1 1.30656296488 False
            A2 1.30656296486 False
            A3 1.30656296475 False
            P12 0.99999999996 False
            P13 0.999999999967 False
            P21 0.999999999967 False
            P23 0.999999999997 False
            P31 0.999999999769 False
            P32 0.999999999839 False

        When, the 1-norm is < 1, the product is pisot::

            sage: it = C.n_matrices_semi_norm_iterator(2, p=1)
            sage: for w,s,b in it: print(w,s,b)  # long time
            A1,A1 1.0 False
            A1,A2 1.0 False
            A1,A3 1.0 False
            A1,P12 0.999998922557 False
            A1,P13 0.999997464905 False
            A1,P21 0.999993244882 False
            A1,P23 0.999999150973 True
            A1,P31 0.999994030522 False
            A1,P32 0.999998046513 True
            A2,A1 1.0 False
            A2,A2 1.0 False
            A2,A3 1.0 False
            A2,P12 0.99999375291 False
            A2,P13 0.999995591588 True
            ...
            P31,A3 0.999988326888 False
            P31,P12 0.749998931902 True
            P31,P23 0.799999157344 True
            P31,P32 0.749993104833 True
            P32,A1 0.999997170005 True
            P32,A3 0.99999420509 False
            P32,P13 0.666665046248 True
            P32,P21 0.666665629351 True
            P32,P31 0.666664488371 True
        """
        if n == 0:
            raise NotImplementedError
        for w,m in self.n_matrices_iterator(n):
            cone = m*self.cone(w[-1])
            yield w, semi_norm_cone(m.transpose(), cone, p=p), self.is_pisot(w)

    def n_matrices_distorsion_iterator(self, n, p=1):
        r"""
        Return the the distorsion of the n-cylinders.

        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: T = cocycles.Sorted_ARP()
            sage: it =T.n_matrices_distorsion_iterator(1)
            sage: list(it)
            [(word: A1, 2),
             (word: A2, 2),
             (word: A3, 2),
             (word: P1, 3),
             (word: P2, 3),
             (word: P3, 3)]
        """
        for w,m in self.n_matrices_iterator(n):
            yield w, distorsion(m, p=p)

    def n_cylinders_iterator(self, n):
        r"""
        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: C = cocycles.ARP()
            sage: it = C.n_cylinders_iterator(1)
            sage: for w,cyl in it: print("{}\n{}".format(w,cyl))
            1
            [1 1 1]
            [0 1 0]
            [0 0 1]
            2
            [1 0 0]
            [1 1 1]
            [0 0 1]
            3
            [1 0 0]
            [0 1 0]
            [1 1 1]
            123
            [1 0 1]
            [1 1 1]
            [1 1 2]
            132
            [1 1 0]
            [1 2 1]
            [1 1 1]
            213
            [1 1 1]
            [0 1 1]
            [1 1 2]
            231
            [2 1 1]
            [1 1 0]
            [1 1 1]
            312
            [1 1 1]
            [1 2 1]
            [0 1 1]
            321
            [2 1 1]
            [1 1 1]
            [1 0 1]
        """
        if n == 0:
            raise NotImplementedError
        for w,m in self.n_matrices_iterator(n):
            yield w, m*self.cone(w[-1])

    def n_cylinders_edges(self, n):
        r"""
        Return the set of edges of the n-cylinders.

        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: ARP = cocycles.ARP()
            sage: ARP.n_cylinders_edges(1)
            {frozenset({(0, 0, 1), (1, 0, 1)}),
             frozenset({(1, 0, 1), (1, 1, 0)}),
             frozenset({(1, 1, 1), (2, 1, 1)}),
             frozenset({(1, 1, 0), (1, 1, 1)}),
             frozenset({(0, 0, 1), (0, 1, 1)}),
             frozenset({(1, 0, 1), (1, 1, 1)}),
             frozenset({(0, 1, 0), (1, 1, 0)}),
             frozenset({(1, 0, 0), (1, 0, 1)}),
             frozenset({(1, 1, 1), (1, 2, 1)}),
             frozenset({(1, 1, 0), (1, 2, 1)}),
             frozenset({(1, 1, 1), (1, 1, 2)}),
             frozenset({(1, 0, 1), (1, 1, 2)}),
             frozenset({(0, 1, 1), (1, 1, 1)}),
             frozenset({(1, 1, 0), (2, 1, 1)}),
             frozenset({(0, 1, 0), (0, 1, 1)}),
             frozenset({(0, 1, 1), (1, 1, 2)}),
             frozenset({(0, 1, 1), (1, 1, 0)}),
             frozenset({(1, 0, 0), (1, 1, 0)}),
             frozenset({(0, 1, 1), (1, 2, 1)}),
             frozenset({(1, 0, 1), (2, 1, 1)}),
             frozenset({(0, 1, 1), (1, 0, 1)})}
        """
        from sage.rings.finite_rings.integer_mod_ring import Integers
        edges = set()
        for w,cyl in self.n_cylinders_iterator(n):
            cols = cyl.columns()
            indices = Integers(len(cols))
            edges.update(frozenset((cols[i], cols[i+1])) for i in indices)
        return edges

    def is_pisot(self, w):
        r"""
        """
        m = self.word_to_matrix(w)
        S = sorted((abs(e) for e in m.eigenvalues()), reverse=True)
        return S[0] > 1 and S[1] < 1

    def non_pisot_automaton(self, n):
        r"""
        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: C = cocycles.ARP()
            sage: A = C.non_pisot_automaton(2)
            sage: A
            Automaton with 2 states
            sage: A.graph().plot(edge_labels=True)   # not tested
        """
        L = []
        for i in range(n):
            L.extend(self.n_matrices_non_pisot(i))
        alphabet = self._language._alphabet
        F = FiniteLanguage(alphabet, L)
        A = F.minimal_automaton()  
        return A
        #G = A.graph()
        #to_remove = set(A.states()) - set(A.final_states())
        #G.delete_vertices(to_remove)
        #return G

    def first_postive_automaton(self, n):
        r"""
        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: C = cocycles.Cassaigne()
            sage: A = C.first_postive_automaton(7)
            sage: A
            Automaton with 21 states
            sage: A.graph().plot(edge_labels=True)   # not tested
        """
        from slabbe.matrices import is_positive
        L = []
        for i in range(n):
            L.extend(w for w,M in self.n_matrices_iterator(i) if is_positive(M))
        alphabet = self._language._alphabet
        F = FiniteLanguage(alphabet, L)
        return F.minimal_automaton()

    def distorsion_max(self, n, p=1):
        r"""
        EXAMPLES:

        Non borné::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: T = cocycles.Sorted_ARP()
            sage: T.distorsion_max(1, p=oo)
            1
            sage: T.distorsion_max(2, p=oo)
            3
            sage: T.distorsion_max(3, p=oo)
            5
            sage: T.distorsion_max(4, p=oo)
            7
        """
        return max(d for (w,d) in self.n_matrices_distorsion_iterator(n, p=p))

    def distorsion_argmax(self, n, p=1):
        r"""
        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: ARP = cocycles.Sorted_ARP()
            sage: ARP.distorsion_argmax(1)
            (
                      [1 0 0]
                      [1 1 0]
            word: A1, [3 2 1]
            )
        """
        it = self.n_cylinders_iterator(n)
        key = lambda w_m:distorsion(w_m[1], p=p)
        return max(it, key=key)

    def plot_n_cylinders(self, n, labels=True):
        r"""
        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: C = cocycles.Sorted_ARP()
            sage: G = C.plot_n_cylinders(3)
        """
        from sage.plot.graphics import Graphics
        from sage.plot.polygon import polygon
        from sage.plot.text import text
        from .matrices import M3to2
        G = Graphics()
        for w,cyl in self.n_cylinders_iterator(n):
            columns = cyl.columns()
            G += polygon((M3to2*col/col.norm(1) for col in columns), fill=False) 
            if labels:
                sum_cols = sum(columns)
                G += text("{}".format(w), M3to2*sum_cols/sum_cols.norm(1))
        return G

    def plot_n_matrices_eigenvectors(self, n, side='right', color_index=0, draw_line=False):
        r"""
        INPUT:

        - ``n`` -- integer, length
        - ``side`` -- ``'left'`` or ``'right'``, drawing left or right
          eigenvectors
        - ``color_index`` -- 0 for first letter, -1 for last letter
        - ``draw_line`` -- boolean

        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: ARP = cocycles.ARP()
            sage: G = ARP.plot_n_matrices_eigenvectors(2)
        """
        from sage.plot.graphics import Graphics
        from sage.plot.point import point
        from sage.plot.line import line
        from sage.plot.text import text
        from sage.plot.colors import hue
        from sage.modules.free_module_element import vector
        from .matrices import M3to2
        R = self.n_matrices_eigenvectors(n)
        L = [(w, M3to2*(a/sum(a)), M3to2*(b/sum(b))) for (w,a,b) in R]
        G = Graphics()
        alphabet = self._language._alphabet
        color_ = dict( (letter, hue(i/float(len(alphabet)))) for i,letter in
                enumerate(alphabet))
        for letter in alphabet:
            L_filtered = [(w,p1,p2) for (w,p1,p2) in L if w[color_index] == letter]
            words,rights,lefts = zip(*L_filtered)
            if side == 'right':
                G += point(rights, color=color_[letter], legend_label=letter)
            elif side == 'left':
                G += point(lefts,  color=color_[letter], legend_label=letter)
            else:
                raise ValueError("side(=%s) should be left or right" % side)

        if draw_line:
            for (a,b) in L:
                G += line([a,b], color='black', linestyle=":")
        G += line([M3to2*vector(a) for a in [(1,0,0), (0,1,0), (0,0,1), (1,0,0)]]) 
        title = "%s eigenvectors, colored by letter w[%s] of cylinder w" % (side, color_index)
        G += text(title, (0.5, 1.05), axis_coords=True)
        G.axes(False)
        return G

    def plot_pisot_conjugates(self, n):
        r"""
        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: B = cocycles.Sorted_Brun()
            sage: G = B.plot_pisot_conjugates(5)   # long time (8s)

        Image envoyee a Timo (6 mai 2014)::

            sage: G = sum(B.plot_pisot_conjugates(i) for i in [1..6])  #not tested
        """
        from sage.plot.point import points
        Lreal = []
        Limag = []
        for w,s in self.n_matrices_eigenvalues_iterator(n):
            a,b,c = sorted(s, key=abs)
            if a.imag() == 0 and b.imag() == 0:
                Lreal.append((a,b))
            else:
                Limag.append((a.real(),a.imag()))
                Limag.append((b.real(),b.imag()))
        return points(Lreal) + points(Limag, color='red')

    def tikz_n_cylinders(self, n, labels=None, scale=1):
        r"""
        INPUT:

        - ``n`` -- integer, for the nth-cylinders
        - ``labels`` -- None, True or False (default: None), if None, it
          takes value True if n is 1.
        - ``scale`` -- real (default: 1), scale value for tikzpicture

        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: ARP = cocycles.ARP()
            sage: t = ARP.tikz_n_cylinders(1, labels=True, scale=4)
            sage: t
            \documentclass[tikz]{standalone}
            \usepackage{amsmath}
            \begin{document}
            \begin{tikzpicture}
            [scale=4]
            \draw (0.0000, 1.0000) -- (-0.4330, 0.2500);
            \draw (0.0000, -0.5000) -- (-0.4330, 0.2500);
            \draw (0.0000, 0.0000) -- (-0.2165, -0.1250);
            ...
            ... 23 lines not printed (1317 characters in total) ...
            ...
            \node at (-0.1443, 0.1667) {$213$};
            \node at (-0.2165, 0.0417) {$231$};
            \node at (0.0722, -0.2083) {$312$};
            \node at (-0.0722, -0.2083) {$321$};
            \end{tikzpicture}
            \end{document}

        ::

            sage: from sage.misc.temporary_file import tmp_filename
            sage: filename = tmp_filename('temp','.pdf')
            sage: _ = t.pdf(filename)
        """
        if labels is None:
            labels = True if n == 1 else False
        lines = []
        lines.append(r"\begin{tikzpicture}")
        lines.append("[scale={}]".format(scale))
        from .matrices import M3to2
        for (u,v) in self.n_cylinders_edges(n):
            u = rounded_string_vector(M3to2 * u / u.norm(1), digits=4)
            v = rounded_string_vector(M3to2 * v / v.norm(1), digits=4)
            lines.append(r"\draw {} -- {};".format(u,v))
        if labels:
            for w,cyl in self.n_cylinders_iterator(n):
                u = sum(c / c.norm(1) for c in cyl.columns())
                u = rounded_string_vector(M3to2 * u / u.norm(1), digits=4)
                lines.append(r"\node at {} {{${}$}};".format(u, w))
        lines.append(r"\end{tikzpicture}")
        from slabbe import TikzPicture
        return TikzPicture("\n".join(lines), use_sage_preamble=False)

#####################
# Predefined Cocycles
#####################
class MatrixCocycleGenerator(object):
    def ARP(self):
        from sage.matrix.constructor import identity_matrix
        A1 = matrix(3, [1,1,1, 0,1,0, 0,0,1])
        A2 = matrix(3, [1,0,0, 1,1,1, 0,0,1])
        A3 = matrix(3, [1,0,0, 0,1,0, 1,1,1])
        P12 = matrix(3, [1,0,1, 1,1,1, 0,0,1])
        P13 = matrix(3, [1,1,0, 0,1,0, 1,1,1])
        P23 = matrix(3, [1,0,0, 1,1,0, 1,1,1])
        P21 = matrix(3, [1,1,1, 0,1,1, 0,0,1])
        P31 = matrix(3, [1,1,1, 0,1,0, 0,1,1])
        P32 = matrix(3, [1,0,0, 1,1,1, 1,0,1])
        gens = (A1, A2, A3, P23, P32, P13, P31, P12, P21)
        alphabet = [1, 2, 3, 123, 132, 213, 231, 312, 321]
        gens = dict(zip(alphabet, gens))

        cone = {}
        cone[123] = H23 = matrix(3, [1,0,1, 0,1,0, 0,0,1])
        cone[132] = H32 = matrix(3, [1,1,0, 0,1,0, 0,0,1])
        cone[213] = H13 = matrix(3, [1,0,0, 0,1,1, 0,0,1])
        cone[231] = H31 = matrix(3, [1,0,0, 1,1,0, 0,0,1])
        cone[312] = H12 = matrix(3, [1,0,0, 0,1,0, 0,1,1])
        cone[321] = H21 = matrix(3, [1,0,0, 0,1,0, 1,0,1])
        cone[1] = cone[2] = cone[3] = identity_matrix(3) 

        from .language import languages
        return MatrixCocycle(gens, cone, language=languages.ARP())

    def ArnouxRauzy(self):
        A1 = matrix(3, [1,1,1, 0,1,0, 0,0,1])
        A2 = matrix(3, [1,0,0, 1,1,1, 0,0,1])
        A3 = matrix(3, [1,0,0, 0,1,0, 1,1,1])
        gens = (A1, A2, A3)
        alphabet = [1, 2, 3]
        gens = dict(zip(alphabet, gens))
        return MatrixCocycle(gens)

    def Sorted_Brun(self):
        B1 = matrix(3, [1,0,0, 0,1,0, 0,1,1])
        B2 = matrix(3, [1,0,0, 0,0,1, 0,1,1])
        B3 = matrix(3, [0,1,0, 0,0,1, 1,0,1])
        gens = (B1, B2, B3)
        alphabet = [1, 2, 3]
        gens = dict(zip(alphabet, gens))
        cone = matrix(3, [1,1,1,0,1,1,0,0,1])
        return MatrixCocycle(gens, cone)

    def Sorted_ARP(self):
        A1 = matrix(3, [1,0,0,0,1,0,-1,-1,1]).inverse()
        A2 = matrix(3, [1,0,0,-1,-1,1,0,1,0]).inverse()
        A3 = matrix(3, [-1,-1,1,1,0,0,0,1,0]).inverse()
        P1 = matrix(3, [0,-1,1,1,0,0,-1,1,0]).inverse()
        P2 = matrix(3, [-1,1,0,0,-1,1,1,0,0]).inverse()
        P3 = matrix(3, [0,-1,1,-1,1,0,1,0,0]).inverse()
        gens = (A1, A2, A3, P1, P2, P3)
        alphabet = ['A1', 'A2', 'A3', 'P1', 'P2', 'P3']
        gens = dict(zip(alphabet, gens))
        cone = matrix(3, [1,0,0,1,1,0,1,1,1])
        return MatrixCocycle(gens, cone)

    def Sorted_ARPMulti(self, order=3):
        A1 = matrix(3, [1,0,0,0,1,0,-1,-1,1]).inverse()
        P1 = matrix(3, [0,-1,1,1,0,0,-1,1,0]).inverse()
        P2 = matrix(3, [-1,1,0,0,-1,1,1,0,0]).inverse()
        P3 = matrix(3, [0,-1,1,-1,1,0,1,0,0]).inverse()
        cone = matrix(3, [1,0,0,1,1,0,1,1,1])
        gens = [P1,P2,P3]
        t12 = matrix(3, [1,0,0,0,0,1,0,1,0])
        t132 = matrix(3, [0,1,0,0,0,1,1,0,0])
        for i in range(1, order+1):
            A1_i = A1**i
            gens.append(A1_i * P1)
            gens.append(A1_i * P2)
            gens.append(A1_i * P3)
            gens.append(A1_i * t12)
            gens.append(A1_i * t132)
        return MatrixCocycle(gens, cone)

    def Cassaigne(self):
        r"""
        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: c = cocycles.Cassaigne()
            sage: list(m for (w,m) in c.n_cylinders_iterator(2))
            [
            [1 1 1]  [1 1 0]  [0 0 1]  [1 0 0]
            [0 1 0]  [0 1 1]  [1 1 0]  [0 1 0]
            [0 0 1], [1 0 0], [0 1 1], [1 1 1]
            ]
        """
        C1 = matrix(3, [1,1,0, 0,0,1, 0,1,0])
        C2 = matrix(3, [0,1,0, 1,0,0, 0,1,1])
        gens = {1:C1, 2:C2}
        return MatrixCocycle(gens)

    def Cassaigne_accelerated(self, order=3):
        r"""
        EXAMPLES::

            sage: from slabbe.matrix_cocycle import cocycles
            sage: c = cocycles.Cassaigne_accelerated(order=3)
            sage: c
            Cocycle with 6 gens over Language of finite words over
            alphabet ['11', '121', '12^{2}1', '212', '21^{2}2', '22']

        """
        C1 = matrix(3, [1,1,0, 0,0,1, 0,1,0])
        C2 = matrix(3, [0,1,0, 1,0,0, 0,1,1])
        C = {1:C1, 2:C2}
        gens = {}
        for i in range(order):
            for (a,b) in [(1,2), (2,1)]:
                if i == 0:
                    code = '{}{}'.format(a,a)
                elif i == 1:
                    code = '{}{}{}'.format(a,b,a)
                else:
                    code = '{}{}^{{{}}}{}'.format(a,b,i,a)
                gens[code] = C[a]*C[b]**i*C[a]
        return MatrixCocycle(gens)

    def Reverse(self):
        A1 = matrix(3, [1,1,1, 0,1,0, 0,0,1])
        A2 = matrix(3, [1,0,0, 1,1,1, 0,0,1])
        A3 = matrix(3, [1,0,0, 0,1,0, 1,1,1])
        R = matrix(3, [0,1,1, 1,0,1, 1,1,0])
        gens = {1:A1, 2:A2, 3:A3, 4:R}
        return MatrixCocycle(gens)

    def Brun(self):
        B12 = matrix(3, [1,0,0, 1,1,0, 0,0,1])
        B13 = matrix(3, [1,0,0, 0,1,0, 1,0,1])
        B21 = matrix(3, [1,1,0, 0,1,0, 0,0,1])
        B23 = matrix(3, [1,0,0, 0,1,0, 0,1,1])
        B31 = matrix(3, [1,0,1, 0,1,0, 0,0,1])
        B32 = matrix(3, [1,0,0, 0,1,1, 0,0,1])
        gens = (B23, B32, B13, B31, B12, B21)
        alphabet = [123, 132, 213, 231, 312, 321]
        gens = dict(zip(alphabet, gens))

        cone = {}
        cone[123] = B12
        cone[132] = B13
        cone[213] = B21
        cone[231] = B23
        cone[312] = B31
        cone[321] = B32

        from .language import languages
        return MatrixCocycle(gens, cone, language=languages.Brun())

    def Selmer(self):
        S123 = matrix(3, [1,0,0, 0,1,0, 1,0,1])
        S132 = matrix(3, [1,0,0, 1,1,0, 0,0,1])
        S213 = matrix(3, [1,0,0, 0,1,0, 0,1,1])
        S231 = matrix(3, [1,1,0, 0,1,0, 0,0,1])
        S312 = matrix(3, [1,0,0, 0,1,1, 0,0,1])
        S321 = matrix(3, [1,0,1, 0,1,0, 0,0,1])
        gens = (S123, S132, S213, S231, S312, S321)
        alphabet = [123, 132, 213, 231, 312, 321]
        gens = dict(zip(alphabet, gens))

        cone = {}
        cone[213] = cone[312] = matrix.column(3, [1,1,1, 1,1,0, 1,0,1])
        cone[123] = cone[321] = matrix.column(3, [1,1,1, 0,1,1, 1,1,0])
        cone[132] = cone[231] = matrix.column(3, [1,1,1, 1,0,1, 0,1,1])

        from .language import languages
        return MatrixCocycle(gens, cone, language=languages.Selmer())

    def Poincare(self):
        P12 = matrix(3, [1,0,1, 1,1,1, 0,0,1])
        P13 = matrix(3, [1,1,0, 0,1,0, 1,1,1])
        P23 = matrix(3, [1,0,0, 1,1,0, 1,1,1])
        P21 = matrix(3, [1,1,1, 0,1,1, 0,0,1])
        P31 = matrix(3, [1,1,1, 0,1,0, 0,1,1])
        P32 = matrix(3, [1,0,0, 1,1,1, 1,0,1])
        gens = (P23, P32, P13, P31, P12, P21)
        alphabet = [123, 132, 213, 231, 312, 321]
        gens = dict(zip(alphabet, gens))
        return MatrixCocycle(gens)

    def FullySubtractive(self):
        F1 = matrix(3, [1,0,0, 1,1,0, 1,0,1])
        F2 = matrix(3, [1,1,0, 0,1,0, 0,1,1])
        F3 = matrix(3, [1,0,1, 0,1,1, 0,0,1])
        gens = (F1, F2, F3)
        alphabet = [1, 2, 3]
        gens = dict(zip(alphabet, gens))
        return MatrixCocycle(gens)

cocycles = MatrixCocycleGenerator()

#####################
# Helper functions
#####################
def rounded_string_vector(v, digits=4):
    r"""
    EXAMPLES::

        sage: from slabbe.matrix_cocycle import rounded_string_vector
        sage: v = (-0.144337567297406, 0.166666666666667)
        sage: rounded_string_vector(v)
        '(-0.1443, 0.1667)'
        sage: rounded_string_vector(v, digits=6)
        '(-0.144338, 0.166667)'
    """
    s = "{{:.{}f}}".format(digits)
    content = ", ".join(s.format(float(a)) for a in v)
    return "({})".format(content)

def distorsion(M, p=1):
    r"""
    1 Avril 2014. L'ancien ratio n'était pas le bon. Je n'utilisais pas les
    bonnes normes.

    EXAMPLES::

        sage: from slabbe.matrix_cocycle import distorsion
        sage: M = matrix(3, (1,2,3,4,5,6,7,8,9))
        sage: M
        [1 2 3]
        [4 5 6]
        [7 8 9]
        sage: distorsion(M)
        3/2
        sage: (3+6+9) / (1+4+7)
        3/2
        sage: distorsion(M, p=oo)
        9/7
    """
    norms = [c.norm(p=p) for c in M.columns()]
    return max(norms) / min(norms)

def is_pisot(m):
    r"""
    EXAMPLES::

        sage: from slabbe.matrix_cocycle import is_pisot
        sage: M = matrix(3, (1,2,3,4,5,6,7,8,9))
        sage: is_pisot(M)
        False
    """
    S = sorted((abs(e) for e in m.eigenvalues()), reverse=True)
    return S[0] > 1 and S[1] < 1

def perron_right_eigenvector(M):
    r"""
    EXAMPLES::

        sage: from slabbe.matrix_cocycle import perron_right_eigenvector
        sage: m = matrix(2,[-11,14,-26,29])
        sage: perron_right_eigenvector(m)           # tolerance 0.00001
        (15.0000000000000, (0.35, 0.6499999999999999))
    """
    from sage.modules.free_module_element import vector
    from sage.rings.real_mpfr import RR
    from sage.rings.all import CC
    import numpy
    eig, vec = numpy.linalg.eig(M)
    index = abs(eig).argmax()
    rightv = vec.transpose()[index]
    if eig[index].imag == 0:
        eig_sage = RR(eig[index].real)
        vec_sage = vector(a.real for a in rightv)
    else:
        eig_sage = CC(eig[index])
        vec_sage = vector(CC, rightv)
    return eig_sage, vec_sage/sum(vec_sage)

def semi_norm_D(v):
    r"""
    EXAMPLES::

        sage: from slabbe.matrix_cocycle import semi_norm_D
        sage: semi_norm_D((1,2,3,-5))
        8

    """
    return max(v)-min(v)

def semi_norm_v(M, v,  p=2, verbose=False):
    r"""
    Return the semi norm on the hyperplane orthogonal to v.

    EXAMPLES::

        sage: from slabbe.matrix_cocycle import semi_norm_v
        sage: A1 = matrix(3, [1,-1,-1, 0,1,0, 0,0,1]).inverse()
        sage: semi_norm_v(A1, vector( (1,1,1)))[0]      # tolerance 0.0001
        0.9999999999890247
        sage: semi_norm_v(A1, vector( (1,1,1)), p=1)[0]   # tolerance 0.0001
        0.9999394820959548
        sage: semi_norm_v(A1, vector( (1,1,1)), p=oo)[0]   # tolerance 0.0001
        1.0

    ::

        sage: m = matrix(3,[0,0,0, 1,0,1, 0,-1,0])
        sage: semi_norm_v(m, vector((1,1,1)), p='D')[0]  # tolerance 0.0001
        0.6666436827952827

    """
    from sage.modules.free_module_element import vector
    from sage.numerical.optimize import minimize_constrained
    def func(z):
        vz = vector(z)
        if p == 'D':
            return - semi_norm_D(M*vz) / semi_norm_D(vz)
        else:
            return - (M*vz).norm(p) / vz.norm(p)
    cons = [lambda z: v * vector(z),
            lambda z: - v * vector(z)]
    x0 = list(range(len(v)))
    x0[0] = v[1]
    x0[1] = -v[0]
    rep = minimize_constrained(func, cons, x0)
    if verbose:
        print(rep, rep.norm(), rep*v)
    return -func(rep), rep

def semi_norm_cone(M, cone,  p=2, verbose=False):
    r"""
    Return the semi norm on the hyperplane orthogonal to v where v lives in
    the cone.

    EXAMPLES:

    For Arnoux-Rauzy, only the 1-norm works::

        sage: from slabbe.matrix_cocycle import semi_norm_cone
        sage: A1 = matrix(3, [1,1,1, 0,1,0, 0,0,1])
        sage: cone = A1
        sage: semi_norm_cone(A1.transpose(), cone, p=1)    # tolerance 0.00001
        0.9999999999999998
        sage: semi_norm_cone(A1.transpose(), cone, p=oo)   # tolerance 0.0001
        1.9999757223144654
        sage: semi_norm_cone(A1.transpose(), cone, p=2)   # tolerance 0.00001
        1.3065629648763757

    For Poincaré, all norms work::

        sage: P21 = matrix(3, [1,1,1, 0,1,1, 0,0,1])
        sage: H21 = matrix(3, [1,0,0, 0,1,0, 1,0,1])
        sage: cone = P21 * H21
        sage: semi_norm_cone(P21.transpose(), cone, p=1)   # tolerance 0.00001
        0.9999957276014074
        sage: semi_norm_cone(P21.transpose(), cone, p=oo)   # tolerance 0.00001
        1.0
        sage: semi_norm_cone(P21.transpose(), cone, p=2)   # tolerance 0.00001
        0.9999999999670175

    For Poincaré on the whole cone, it works for some norms::

        sage: P21 = matrix(3, [1,1,1, 0,1,1, 0,0,1])
        sage: cone = P21
        sage: semi_norm_cone(P21.transpose(), cone, p=1)   # tolerance 0.0001 # known bug
        1.9999675644077723
        sage: semi_norm_cone(P21.transpose(), cone, p=2)   # tolerance 0.00001
        1.6180339887021953
        sage: semi_norm_cone(P21.transpose(), cone, p=oo)   # tolerance 0.00001
        1.0

    For a product, all norms work::

        sage: A1 = matrix(3, [1,1,1, 0,1,0, 0,0,1])
        sage: P21 = matrix(3, [1,1,1, 0,1,1, 0,0,1])
        sage: H21 = matrix(3, [1,0,0, 0,1,0, 1,0,1])
        sage: M = A1 * P21
        sage: cone = A1 * P21 * H21
        sage: semi_norm_cone(M.transpose(), cone, p=1)   # tolerance 0.00001
        0.999993244882415
        sage: semi_norm_cone(M.transpose(), cone, p=oo)   # tolerance 0.00001
        0.9999935206958908
        sage: semi_norm_cone(M.transpose(), cone, p=2)   # tolerance 0.00001
        0.7529377601317161

    ::

        sage: M = cone = matrix(3,[2,3,2, 2,2,1, 1,2,1])
        sage: semi_norm_cone(M.T, cone, p='D')  # tolerance 0.00001
        0.7499977852638109

    """
    from sage.modules.free_module_element import vector
    from sage.numerical.optimize import minimize_constrained
    a,b,c = cone.columns()
    ab = vector(matrix((a,b)).right_kernel_matrix().row(0))
    ac = vector(matrix((a,c)).right_kernel_matrix().row(0))
    bc = vector(matrix((b,c)).right_kernel_matrix().row(0))
    middle = a+b+c
    cons = []
    if ab * middle < 0:
        cons.append(lambda z: -ab * vector(z))
    else:
        cons.append(lambda z: ab * vector(z))
    if ac * middle < 0:
        cons.append(lambda z: -ac * vector(z))
    else:
        cons.append(lambda z: ac * vector(z))
    if bc * middle < 0:
        cons.append(lambda z: -bc * vector(z))
    else:
        cons.append(lambda z: bc * vector(z))
    if not all(con(middle) > 0 for con in cons):
        raise ValueError("the middle(={}) should be in the cone".format(middle))
    func = lambda v : - semi_norm_v(M,vector(v),p)[0]
    x0 = middle
    rep = minimize_constrained(func, cons, x0)
    if not all((con(rep) >= 0 or abs(con(rep)) < 1e-7) for con in cons):
        raise ValueError("the answer (={}) should be in the cone".format(rep))
    if not all(r >= 0 or abs(r) < 1e-7 for r in rep):
        raise ValueError("the answer (={}) should be positive".format(rep))
    if verbose:
        print("optimal found at v = {}".format(rep / rep.norm(1)))
        val,z = semi_norm_v(M,rep,p,verbose=False)
        z /= z.norm(1)
        print("vector z (orthogonal to v) = {}".format(z.n(15)))
        print("||M z|| / ||z|| = {}".format(val))
    return -func(rep)

####################
# Polyhedron Partition
####################
def arp_polyhedron(d=3):
    r"""
    Return the d-dimensional 1-cylinders of the ARP algorithm.

    EXAMPLES::

        sage: from slabbe.matrix_cocycle import arp_polyhedron
        sage: A,P,L = arp_polyhedron(3)
        sage: A.vertices_list()
        [[0, 0, 0], [1/2, 1/2, 0], [1/2, 1/4, 1/4], [1, 0, 0]]
        sage: P.vertices_list()
        [[0, 0, 0], [1/2, 1/2, 0], [1/2, 1/4, 1/4], [1/3, 1/3, 1/3]]

    ::

        sage: A,P,L = arp_polyhedron(4)
        sage: A.vertices_list()
        [[0, 0, 0, 0],
         [1/2, 1/2, 0, 0],
         [1/2, 1/6, 1/6, 1/6],
         [1/2, 1/4, 1/4, 0],
         [1, 0, 0, 0]]
        sage: P.vertices_list()
        [[0, 0, 0, 0],
         [1/2, 1/2, 0, 0],
         [1/2, 1/4, 1/4, 0],
         [1/2, 1/6, 1/6, 1/6],
         [1/4, 1/4, 1/4, 1/4],
         [1/3, 1/3, 1/3, 0]]

    ::

        sage: A,P,L = arp_polyhedron(5)
        sage: A.vertices_list()
        [[0, 0, 0, 0, 0],
         [1/2, 1/2, 0, 0, 0],
         [1/2, 1/8, 1/8, 1/8, 1/8],
         [1/2, 1/6, 1/6, 1/6, 0],
         [1/2, 1/4, 1/4, 0, 0],
         [1, 0, 0, 0, 0]]
        sage: P.vertices_list()
        [[0, 0, 0, 0, 0],
         [1/2, 1/2, 0, 0, 0],
         [1/2, 1/6, 1/6, 1/6, 0],
         [1/2, 1/8, 1/8, 1/8, 1/8],
         [1/2, 1/4, 1/4, 0, 0],
         [1/3, 1/3, 1/3, 0, 0],
         [1/5, 1/5, 1/5, 1/5, 1/5],
         [1/4, 1/4, 1/4, 1/4, 0]]
    """
    from sage.geometry.polyhedron.constructor import Polyhedron
    positive = [ [0]*i + [1] + [0]*(d-i) for i in range(1, d+1)]
    atmostone = [[1] + [-1]*d]
    ieq_arnoux = [[0]+[1]+[-1]*(d-1)]
    ieq_arnoux_not = [[0]+[-1]+[1]*(d-1)]
    ieq_sorted = [ [0]*i + [1,-1] + [0]*(d-i-1) for i in range(1,d)]
    A = Polyhedron(ieqs=positive + atmostone + ieq_sorted + ieq_arnoux)
    P = Polyhedron(ieqs=positive + atmostone + ieq_sorted + ieq_arnoux_not)
    L = Polyhedron(ieqs=positive + atmostone + ieq_sorted)
    return A,P,L

def cassaigne_polyhedron(d=3):
    r"""
    Return the d-dimensional 1-cylinders of the Cassaigne algorithm.

    (of the dual!)

    EXAMPLES::

        sage: from slabbe.matrix_cocycle import cassaigne_polyhedron
        sage: L,La,Lb = cassaigne_polyhedron(3)
        sage: L.vertices_list()
        [[0, 0, 0], [0, 1/2, 1/2], [1/3, 1/3, 1/3], [1/2, 1/2, 0]]
        sage: La.vertices_list()
        [[0, 0, 0], [0, 1/2, 1/2], [1/3, 1/3, 1/3], [1/4, 1/2, 1/4]]
        sage: Lb.vertices_list()
        [[0, 0, 0], [1/3, 1/3, 1/3], [1/2, 1/2, 0], [1/4, 1/2, 1/4]]

    ::

        sage: L,La,Lb = cassaigne_polyhedron(4)
        sage: L.vertices_list()
        [[0, 0, 0, 0],
         [0, 1/3, 1/3, 1/3],
         [1/3, 1/3, 1/3, 0],
         [1/4, 1/4, 1/4, 1/4],
         [1/5, 2/5, 1/5, 1/5],
         [1/5, 1/5, 2/5, 1/5]]

    ::

        sage: L,La,Lb = cassaigne_polyhedron(5)
        sage: L.vertices_list()
        [[0, 0, 0, 0, 0],
         [0, 1/4, 1/4, 1/4, 1/4],
         [1/4, 1/4, 1/4, 1/4, 0],
         [1/6, 1/6, 1/3, 1/6, 1/6],
         [1/5, 1/5, 1/5, 1/5, 1/5],
         [1/6, 1/3, 1/6, 1/6, 1/6],
         [1/7, 2/7, 2/7, 1/7, 1/7],
         [1/7, 2/7, 1/7, 2/7, 1/7],
         [1/7, 1/7, 2/7, 2/7, 1/7],
         [1/6, 1/6, 1/6, 1/3, 1/6]]
    """
    from sage.geometry.polyhedron.constructor import Polyhedron
    # [-1,7,3,4] represents the inequality 7x_1+3x_2+4x_3>= 1.
    positive = [ [0]*i + [1] + [0]*(d-i) for i in range(1, d+1)]
    atmostone = [[1] + [-1]*d]
    ai_lt_a1d = [[0]+[1]+[0]*(i-2)+[-1]+[0]*(d-i-1)+[1] for i in range(2,d)]
    ai_gt_a1 = [[0]+[-1]+[0]*(i-2)+[1]+[0]*(d-i-1)+[0] for i in range(2,d)]
    ai_gt_ad = [[0]+[0]+[0]*(i-2)+[1]+[0]*(d-i-1)+[-1] for i in range(2,d)]
    a1_gt_ad = [[0]+[1]+[0]*(d-2)+[-1]]
    a1_lt_ad = [[0]+[-1]+[0]*(d-2)+[1]]
    L = Polyhedron(ieqs=positive + atmostone + ai_lt_a1d + ai_gt_a1 + ai_gt_ad)
    La = Polyhedron(ieqs=positive+atmostone+ai_lt_a1d+ai_gt_a1+ai_gt_ad+a1_lt_ad)
    Lb = Polyhedron(ieqs=positive+atmostone+ai_lt_a1d+ai_gt_a1+ai_gt_ad+a1_gt_ad)
    return L, La, Lb




# -*- coding: utf-8 -*-
r"""
André Joyal's Bijection

Problem suggested by Doron Zeilberger during a talk done at CRM, Montreal,
May 11th, 2012 to compare code in different languages. This is a
implementation of the `Joyal's Bijection`__ using Sage. It will not win for
the most brief code, but it is object oriented, documented, reusable,
testable and allows introspection.

__ http://fr.wikipedia.org/wiki/Bijection_de_Joyal

AUTHOR:

    - Sébastien Labbé, May 12th 2012

TODO:

 - Base Endofunction class on sage's FiniteSetMap classes (for both element
   and parent)

EXAMPLES:

Creation of an endofunction
---------------------------

::

    sage: from slabbe import Endofunction
    sage: L = [7, 0, 6, 1, 4, 7, 2, 1, 5] 
    sage: f = Endofunction(L)
    sage: f
    Endofunction:
    [0..8] -> [7, 0, 6, 1, 4, 7, 2, 1, 5] 

Creation of a double rooted tree
--------------------------------

::

    sage: from slabbe import DoubleRootedTree
    sage: L = [(0,6),(2,1),(3,1),(4,2),(5,7),(6,4),(7,0),(8,5)]
    sage: D = DoubleRootedTree(L, 1, 7)
    sage: D
    Double rooted tree:
    Edges: [(0, 6), (2, 1), (3, 1), (4, 2), (5, 7), (6, 4), (7, 0), (8, 5)]
    RootA: 1
    RootB: 7

Joyal's bijection
-----------------

From the endofunction ``f``, we get a double rooted tree::

    sage: f.to_double_rooted_tree()
    Double rooted tree:
    Edges: [(0, 6), (2, 1), (3, 1), (4, 2), (5, 7), (6, 4), (7, 0), (8, 5)]
    RootA: 1
    RootB: 7

From the double rooted tree ``D``, we get an endofunction::

    sage: D.to_endofunction()
    Endofunction:
    [0..8] -> [7, 0, 6, 1, 4, 7, 2, 1, 5]

In fact, we got ``D`` from ``f`` and vice versa::

    sage: D == f.to_double_rooted_tree()
    True
    sage: f == D.to_endofunction()
    True

Endofunctions are defined on the set [0, 1, ..., n-1]
-----------------------------------------------------

As of now, the code supports only endofunctions defined on the set 
[0, 1, ..., n-1] ::

    sage: L = [1, 0, 3, 4, 5, 7, 1]
    sage: f = Endofunction(L)
    Traceback (most recent call last):
    ...
    ValueError: images of [0..6] must be 0 <= i < 7

Another example
---------------

From a list ``L``, we create an endofunction ``f`` ::

    sage: L = [12, 7, 8, 3, 3, 11, 11, 9, 5, 12, 0, 10, 9]
    sage: f = Endofunction(L)
    sage: f
    Endofunction:
    [0..12] -> [12, 7, 8, 3, 3, 11, 11, 9, 5, 12, 0, 10, 9]

From ``f``, we create a double rooted tree ``D``::

    sage: D = f.to_double_rooted_tree(); D
    Double rooted tree:
    Edges: [(0, 12), (1, 7), (2, 8), (3, 12), (4, 3), (5, 11), 
    (6, 11), (7, 9), (8, 5), (10, 0), (11, 10), (12, 9)]
    RootA: 9
    RootB: 3

And from ``D``, we create an endofunction::

    sage: D.to_endofunction()
    Endofunction:
    [0..12] -> [12, 7, 8, 3, 3, 11, 11, 9, 5, 12, 0, 10, 9]

We test that we recover the initial endofunction ``f``::

    sage: f == f.to_double_rooted_tree().to_endofunction()
    True

A random example
----------------

We define the set of all endofunctions on [0..7]::

    sage: from slabbe import Endofunctions
    sage: E = Endofunctions(8)
    sage: E
    Endofunctions of [0..7]

We choose a random endofunction on the set [0..7]::

    sage: f = E.random_element()
    sage: f                               # random
    Endofunction:
    [0..7] -> [5, 5, 0, 4, 5, 0, 1, 1]

We construct a double rooted tree from it::

    sage: f.to_double_rooted_tree()       # random
    Double rooted tree:
    Edges: [(1, 5), (2, 0), (3, 4), (4, 5), (5, 0), (6, 1), (7, 1)]
    RootA: 0
    RootB: 5

We recover an endofunction from the double rooted tree::

    sage: f.to_double_rooted_tree().to_endofunction()   # random
    Endofunction:
    [0..7] -> [5, 5, 0, 4, 5, 0, 1, 1]

Finally, we check the bijection::

    sage: f == f.to_double_rooted_tree().to_endofunction()
    True

Large random example
--------------------

::

    sage: E = Endofunctions(1000)
    sage: f = E.random_element()
    sage: f == f.to_double_rooted_tree().to_endofunction()
    True

TESTS:

We test the limit cases::

    sage: f = Endofunction([0])
    sage: f == f.to_double_rooted_tree().to_endofunction()
    True
    sage: f = Endofunction([0,1])
    sage: f == f.to_double_rooted_tree().to_endofunction()
    True
    sage: f = Endofunction([1,0])
    sage: f == f.to_double_rooted_tree().to_endofunction()
    True

More extensively::

    sage: E = Endofunctions(1)
    sage: all(f == f.to_double_rooted_tree().to_endofunction() for f in E)
    True
    sage: E = Endofunctions(2)
    sage: all(f == f.to_double_rooted_tree().to_endofunction() for f in E)
    True
    sage: E = Endofunctions(3)
    sage: all(f == f.to_double_rooted_tree().to_endofunction() for f in E)
    True
    sage: E = Endofunctions(4)
    sage: all(f == f.to_double_rooted_tree().to_endofunction() for f in E)
    True

TIMING TESTS:

When the extension of the file is .sage::

    sage: E = Endofunctions(3)
    sage: time all(f == f.to_double_rooted_tree().to_endofunction() for f in E) # not tested
    True
    Time: CPU 0.02 s, Wall: 0.02 s
    sage: E = Endofunctions(4)
    sage: time all(f == f.to_double_rooted_tree().to_endofunction() for f in E) # not tested
    True
    Time: CPU 0.22 s, Wall: 0.22 s
    sage: E = Endofunctions(5)
    sage: time all(f == f.to_double_rooted_tree().to_endofunction() for f in E) # not tested
    True
    Time: CPU 2.82 s, Wall: 2.82 s
    sage: E = Endofunctions(6)
    sage: time all(f == f.to_double_rooted_tree().to_endofunction() for f in E) # not tested
    True
    Time: CPU 45.66 s, Wall: 45.74 s

When the extension of the file is .spyx::

    sage: E = Endofunctions(3)
    sage: time all(f == f.to_double_rooted_tree().to_endofunction() for f in E) # not tested
    True
    Time: CPU 0.02 s, Wall: 0.02 s
    sage: E = Endofunctions(4)
    sage: time all(f == f.to_double_rooted_tree().to_endofunction() for f in E) # not tested
    True
    Time: CPU 0.21 s, Wall: 0.21 s
    sage: E = Endofunctions(5)
    sage: time all(f == f.to_double_rooted_tree().to_endofunction() for f in E) # not tested
    True
    Time: CPU 2.71 s, Wall: 2.72 s
    sage: E = Endofunctions(6)
    sage: time all(f == f.to_double_rooted_tree().to_endofunction() for f in E) # not tested
    True
    Time: CPU 44.08 s, Wall: 44.17 s

When the extension of the file is .sage::

    sage: E = Endofunctions(1000)
    sage: f = E.random_element()
    sage: time f == f.to_double_rooted_tree().to_endofunction() # not tested
    True
    Time: CPU 0.09 s, Wall: 0.09 s
    sage: E = Endofunctions(10000)
    sage: f = E.random_element()
    sage: time f == f.to_double_rooted_tree().to_endofunction()  # not tested
    True
    Time: CPU 2.23 s, Wall: 2.24 s

When the extension of the file is .spyx::

    sage: E = Endofunctions(1000)
    sage: f = E.random_element()
    sage: time f == f.to_double_rooted_tree().to_endofunction() # not tested
    True
    Time: CPU 0.11 s, Wall: 0.11 s
    sage: E = Endofunctions(10000)
    sage: f = E.random_element()
    sage: time f == f.to_double_rooted_tree().to_endofunction() # not tested
    True
    Time: CPU 2.91 s, Wall: 2.93 s

"""
#*****************************************************************************
#       Copyright (C) 2008 Sebastien Labbe <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
from copy import copy
from sage.misc.flatten import flatten
from sage.graphs.graph import Graph
from sage.misc.prandom import randint
from sage.sets.finite_set_maps import FiniteSetMaps
from sage.combinat.words.word import Word


class Endofunctions(object):
    r"""
    Returns the set of all endofunction on the set [0..n-1].

    INPUT:

    - ``n`` - positive integer

    EXAMPLES::

        sage: from slabbe import Endofunctions
        sage: Endofunctions(10)
        Endofunctions of [0..9]
    """
    def __init__(self, n):
        r"""
        EXAMPLES::

            sage: from slabbe import Endofunctions
            sage: Endofunctions(10)
            Endofunctions of [0..9]
        """
        self._n = n

    def __repr__(self):
        r"""
        EXAMPLES::

            sage: from slabbe import Endofunctions
            sage: Endofunctions(1)
            Endofunctions of [0..0]
            sage: Endofunctions(10)
            Endofunctions of [0..9]
            sage: Endofunctions(0)
            Endofunctions of [0..-1]
        """
        return "Endofunctions of [0..%s]" % (self._n-1)

    def __iter__(self):
        for f in FiniteSetMaps(self._n):
            yield Endofunction(f)

    def random_element(self):
        r"""
        Return a random endofunction on [0..n-1].

        EXAMPLES::

            sage: from slabbe import Endofunctions
            sage: E = Endofunctions(10)
            sage: E.random_element()          # random
            Endofunction:
            [0..9] -> [2, 8, 7, 0, 0, 6, 2, 3, 5, 9]
            sage: E.random_element()          # random
            Endofunction:
            [0..9] -> [8, 7, 7, 5, 4, 1, 0, 3, 8, 6]
        """
        L = [randint(0,self._n-1) for a in range(self._n)]
        return Endofunction(L)


class Endofunction(object):
    r"""
    Returns an endofunction.

    INPUT:

    - ``L`` - list of length n containing images of the integers from 0 to n-1
      where the images belong to the integers from 0 to n-1.

    EXAMPLES::

        sage: from slabbe import Endofunction
        sage: L = [0, 5, 7, 2, 1, 1, 2, 6, 2, 4]
        sage: f = Endofunction(L)
        sage: f
        Endofunction:
        [0..9] -> [0, 5, 7, 2, 1, 1, 2, 6, 2, 4]
    """
    def __init__(self, L):
        r"""
        """
        if not isinstance(L, list):
            L = list(L)
        len_L = len(L)
        if not all(0 <= i < len_L for i in L):
            raise ValueError("images of [0..%s] must be 0 <= i < %s" % (len_L-1,len_L) )
        self._list = L

    def __repr__(self):
        r"""
        """
        s = "Endofunction:\n"
        s += "[0..%s] -> %s" % (len(self)-1, self._list)
        return s

    def __len__(self):
        r"""
        EXAMPLES::

            sage: from slabbe import Endofunction
            sage: L = [1, 6, 5, 1, 6, 5, 0]
            sage: f = Endofunction(L)
            sage: len(f)
            7
        """
        return len(self._list)

    def __call__(self, a):
        r"""
        EXAMPLES::

            sage: from slabbe import Endofunction
            sage: L = [1, 6, 5, 1, 6, 5, 0]
            sage: f = Endofunction(L)
            sage: f(2)
            5
        """
        return self._list[a]

    def __eq__(self, other):
        r"""
        EXAMPLES::

            sage: from slabbe import Endofunction
            sage: L = [1, 6, 5, 1, 6, 5, 0]
            sage: f = Endofunction(L)
            sage: f == f
            True
            sage: f == 2
            False
        """
        return (isinstance(other, Endofunction) and
                self._list == other._list)

    def __ne__(self, other):
        return not self.__eq__(other)

    def two_cycle_elements(self):
        r"""
        Iterator over elements in a two-cycle.

        EXAMPLES::

            sage: from slabbe import Endofunction
            sage: L = [0, 5, 7, 2, 1, 1, 2, 6, 2, 4]
            sage: f = Endofunction(L)
            sage: list(f.two_cycle_elements())
            [1, 5]
        """
        for i,a in enumerate(self._list):
            if i != a and i == self._list[a] :
                yield i

    def cycle_elements(self):
        r"""
        Returns the list of all elements in a cycle for this endofunction.

        OUTPUT:

        list

        EXAMPLES::

            sage: from slabbe import Endofunction
            sage: L = [6, 5, 7, 2, 1, 1, 2, 6, 2, 4]
            sage: f = Endofunction(L)
            sage: f.cycle_elements()   # random order
            [0, 6, 7, 2, 1, 5]

        .. NOTE::

            ``G.cycle_basis()`` is not implemented for directed or
            multiedge graphs (in Networkx).  Hence, the ``cycle_basis``
            method is missing the 2-cycles.
        """
        G = Graph(loops=True)
        for i, a in enumerate(self._list):
            G.add_edge(i,a)
        C = G.cycle_basis()
        L = flatten(C)
        # adding the missing 2-cycle elements
        for i in self.two_cycle_elements():
            L.append(i)
        return L

    def skeleton(self):
        r"""
        Return the skeleton of the endofunction.

        OUTPUT:

        list

        EXAMPLES::

            sage: from slabbe import Endofunction
            sage: L = [0, 5, 7, 2, 1, 1, 2, 6, 2, 4]
            sage: f = Endofunction(L)
            sage: f.skeleton()
            [0, 5, 7, 1, 2, 6]
        """
        sorted_cycle_elements = sorted(self.cycle_elements())
        L = []
        for a in sorted_cycle_elements:
            L.append(self._list[a])
        return L

    def to_double_rooted_tree(self):
        r"""
        Return the double rooted tree following André Joyal Bijection.

        OUTPUT:

        Double rooted tree

        EXAMPLES::

            sage: from slabbe import Endofunction
            sage: L = [0, 5, 7, 2, 1, 1, 2, 6, 2, 4]
            sage: f = Endofunction(L)
            sage: f.to_double_rooted_tree()
            Double rooted tree:
            Edges: [(0, 5), (1, 2), (2, 6), (3, 2), (4, 1), (5, 7), (7, 1), (8, 2), (9, 4)]
            RootA: 6
            RootB: 0
        """
        tree_list = copy(self._list)
        skeleton = self.skeleton()
        rootA = skeleton[-1]
        rootB = skeleton[0]
        for i in range(len(skeleton)-1):
            tree_list[skeleton[i]] = skeleton[i+1]
        edges = []
        for i,a in enumerate(tree_list):
            if i != rootA:
                edges.append((i,a))
        return DoubleRootedTree(edges, rootA, rootB)

class DoubleRootedTree(object):
    r"""
    Returns a double rooted tree.

    INPUT:

    - ``edges`` - list of edges
    - ``rootA`` - root A
    - ``rootB`` - root B

    EXAMPLES::

        sage: from slabbe import DoubleRootedTree
        sage: edges = [(0,5),(1,2),(2,6),(3,2),(4,1),(5,7),(7,1),(8,2),(9,4)]
        sage: D = DoubleRootedTree(edges, 6, 0)
        sage: D
        Double rooted tree:
        Edges: [(0, 5), (1, 2), (2, 6), (3, 2), (4, 1), (5, 7), (7, 1), (8, 2), (9, 4)]
        RootA: 6
        RootB: 0
    """
    def __init__(self, edges, rootA, rootB):
        r"""
        EXAMPLES::

            sage: from slabbe import DoubleRootedTree
            sage: edges = [(0,5),(1,2),(2,6),(3,2),(4,1),(5,7),(7,1),(8,2),(9,4)]
            sage: D = DoubleRootedTree(edges, 6, 0)
            sage: D
            Double rooted tree:
            Edges: [(0, 5), (1, 2), (2, 6), (3, 2), (4, 1), (5, 7), (7, 1), (8, 2), (9, 4)]
            RootA: 6
            RootB: 0
        """
        self._edges = edges
        self._rootA = rootA
        self._rootB = rootB
        #
        self._graph = None

    def __repr__(self):
        r"""
        EXAMPLES::

            sage: from slabbe import DoubleRootedTree
            sage: edges = [(0,5),(1,2),(2,6),(3,2),(4,1),(5,7),(7,1),(8,2),(9,4)]
            sage: D = DoubleRootedTree(edges, 6, 0)
            sage: D
            Double rooted tree:
            Edges: [(0, 5), (1, 2), (2, 6), (3, 2), (4, 1), (5, 7), (7, 1), (8, 2), (9, 4)]
            RootA: 6
            RootB: 0
        """
        s = "Double rooted tree:\n"
        s += "Edges: %s\n" % self._edges
        s += "RootA: %s\n" % self._rootA
        s += "RootB: %s" % self._rootB
        return s

    def __eq__(self, other):
        r"""
        EXAMPLES::

            sage: from slabbe import DoubleRootedTree
            sage: edges = [(0,5),(1,2),(2,6),(3,2),(4,1),(5,7),(7,1),(8,2),(9,4)]
            sage: D = DoubleRootedTree(edges, 6, 0)
            sage: D == 3
            False
            sage: D == D
            True
        """
        return (isinstance(other, DoubleRootedTree) and
                self._rootA == other._rootA and
                self._rootB == other._rootB and
                set(self._edges) == set(other._edges))

    def __ne__(self, other):
        r"""
        EXAMPLES::

            sage: from slabbe import DoubleRootedTree
            sage: edges = [(0,5),(1,2),(2,6),(3,2),(4,1),(5,7),(7,1),(8,2),(9,4)]
            sage: D = DoubleRootedTree(edges, 6, 0)
            sage: D != 3
            True
            sage: D != D
            False
        """
        return not self.__eq__(other)

    def graph(self):
        r"""
        EXAMPLES::

            sage: from slabbe import DoubleRootedTree
            sage: edges = [(0,5),(1,2),(2,6),(3,2),(4,1),(5,7),(7,1),(8,2),(9,4)]
            sage: D = DoubleRootedTree(edges, 6, 0)
            sage: D.graph()
            Graph on 10 vertices
        """
        if self._graph is None:
            G = Graph()
            G.add_vertex(self._rootA)
            G.add_vertex(self._rootB)
            for a,b in self._edges:
                G.add_edge(a,b)
            self._graph = G
        return self._graph

    def skeleton(self):
        r"""
        EXAMPLES::

            sage: from slabbe import DoubleRootedTree
            sage: edges = [(0,5),(1,2),(2,6),(3,2),(4,1),(5,7),(7,1),(8,2),(9,4)]
            sage: D = DoubleRootedTree(edges, 6, 0)
            sage: D.skeleton()
            [0, 5, 7, 1, 2, 6]
        """
        G = self.graph()
        return G.shortest_path(self._rootB, self._rootA)

    def skeleton_cycles(self):
        r"""
        EXAMPLES::

            sage: from slabbe import DoubleRootedTree
            sage: edges = [(0,5),(1,2),(2,6),(3,2),(4,1),(5,7),(7,1),(8,2),(9,4)]
            sage: D = DoubleRootedTree(edges, 6, 0)
            sage: D.skeleton()
            [0, 5, 7, 1, 2, 6]
            sage: D.skeleton_cycles()
            [(0,), (1, 5), (2, 7, 6)]
        """
        skeleton = self.skeleton()
        sorted_skeleton = sorted(skeleton)
        w = Word(skeleton)
        cycles_standard = w.standard_permutation().to_cycles()
        cycles = []
        for cycle_standard in cycles_standard:
            t = tuple(sorted_skeleton[c-1] for c in cycle_standard)
            cycles.append(t)
        return cycles

    def to_endofunction(self):
        r"""

        EXAMPLES::

            sage: from slabbe import DoubleRootedTree
            sage: edges = [(0,5),(1,2),(2,6),(3,2),(4,1),(5,7),(7,1),(8,2),(9,4)]
            sage: D = DoubleRootedTree(edges, 6, 0)
            sage: D.to_endofunction()
            Endofunction:
            [0..9] -> [0, 5, 7, 2, 1, 1, 2, 6, 2, 4]

        TESTS::

            sage: D = DoubleRootedTree([], 0, 0)
            sage: D.to_endofunction()
            Endofunction:
            [0..0] -> [0]
        """
        function = {}
        # define the function for elements in the skeleton
        for cycle in self.skeleton_cycles():
            len_cycle = len(cycle)
            for i in range(len_cycle):
                a = cycle[i]
                b = cycle[(i+1) % len_cycle]
                function[a] = b
        # delete edges of the graph between elements of the skeleton
        skeleton = self.skeleton()
        G = self.graph()
        for i in range(len(skeleton)-1):
            a = skeleton[i]
            b = skeleton[i+1]
            G.delete_edge(a, b)
        # define the function for all other elements
        todo = skeleton
        while todo:
            a = todo.pop()
            for b in G[a]:
                function[b] = a
                G.delete_edge(a, b)
                todo.append(b)
        # reset graph attribute since we destroyed it
        self._graph = None
        # turn the function (dict) into a list
        function_list = [function[i] for i in range(len(function))]
        return Endofunction(function_list)



# -*- coding: utf-8 -*-
r"""
Christoffel graph

This module was developped for the article on a d-dimensional extension of
Christoffel Words written with Christophe Reutenauer [LR2014]_.

.. [LR2014] Labbé, Sébastien, and Christophe Reutenauer. A d-dimensional Extension of
   Christoffel Words. arXiv:1404.4021__ (April 15, 2014).

__ http://arxiv.org/abs/1404.4021

EXAMPLES:

Christoffel graph in 2d (tikz code)::

    sage: from slabbe import ChristoffelGraph, DiscreteBox
    sage: C = ChristoffelGraph((2,5))
    sage: b = DiscreteBox([-5,5],[-5,5])
    sage: I = C & b
    sage: point_kwds = {'label':lambda p:C.level_value(p),'label_pos':'above right'}
    sage: tikz = I.tikz_noprojection(scale=0.8,point_kwds=point_kwds)

Christoffel graph in 3d (tikz code)::

    sage: C = ChristoffelGraph((2,3,5))
    sage: tikz = C.tikz_kernel()

TODO:

    - Clean kernel_vector method of ChristoffelGraph

"""
#*****************************************************************************
#       Copyright (C) 2013-2014 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
import itertools
from copy import copy
from sage.modules.free_module_element import vector
from sage.matrix.constructor import matrix
from sage.rings.integer_ring import ZZ
from sage.rings.real_mpfr import RR
from sage.functions.other import abs
from slabbe.discrete_subset import DiscreteSubset, DiscreteTube
from slabbe.discrete_plane import DiscretePlane 
from slabbe.matrices import M3to2, M4to3
################################################
# Christoffel Graph
################################################
class ChristoffelGraph(DiscreteSubset):
    r"""
    Subset of a discrete object such that its projection by a matrix is
    inside a certain box.

    INPUT:

    - ``v`` - vector, normal vector

    EXAMPLES::

        sage: from slabbe import ChristoffelGraph
        sage: ChristoffelGraph((2,5))
        Christoffel set of edges for normal vector v=(2, 5)

    ::

        sage: C = ChristoffelGraph((2,5))
        sage: it = C.edges_iterator()
        sage: next(it)
        ((0, 0), (1, 0))

    ::

        sage: C = ChristoffelGraph((2,5,8))
        sage: it = C.edges_iterator()
        sage: next(it)
        ((0, 0, 0), (1, 0, 0))

    ::

        sage: from slabbe import DiscreteBox
        sage: C = ChristoffelGraph((2,5))
        sage: b = DiscreteBox([-5,5],[-5,5])
        sage: I = C & b
        sage: point_kwds = {'label':lambda p:C.level_value(p),'label_pos':'above right'}
        sage: tikz = I.tikz_noprojection(scale=0.8,point_kwds=point_kwds)

    TEST:

    This was once a bug. We make sure it is fixed::

        sage: from slabbe import DiscreteSubset
        sage: C = ChristoffelGraph((2,3,5))
        sage: isinstance(C, DiscreteSubset)
        True

    """
    def __init__(self, v, mod=None):
        r"""
        Constructor.

        Return the Christoffel set of edges.

        EXAMPLES::

            sage: from slabbe import ChristoffelGraph
            sage: ChristoffelGraph((2,5))
            Christoffel set of edges for normal vector v=(2, 5)

        """
        self._v = vector(v)
        if mod:
            self._sum_v = mod
        else:
            self._sum_v = sum(abs(a) for a in v)
        DiscreteSubset.__init__(self, dimension=len(v), edge_predicate=self.has_edge)

    def level_value(self, p):
        r"""
        Return the level value of a point p.

        INPUT:

        - ``p`` - point in the space

        EXAMPLES::

            sage: from slabbe import ChristoffelGraph
            sage: C = ChristoffelGraph((2,5,8))
            sage: C.level_value(vector((2,3,4)))
            6
            sage: C.level_value(vector((1,1,1)))
            0
        """
        return p.dot_product(self._v) % self._sum_v

    def has_edge(self, p, s):
        r"""
        Returns whether it has the edge (p, s) where s-p is a canonical
        vector.

        INPUT:

        - ``p`` - point in the space
        - ``s`` - point in the space

        EXAMPLES::

            sage: from slabbe import ChristoffelGraph
            sage: C = ChristoffelGraph((2,5,8))
            sage: C.has_edge(vector((0,0,0)), vector((0,0,1)))
            True
            sage: C.has_edge(vector((0,0,0)), vector((0,0,2)))
            False
            sage: C.has_edge(vector((0,0,0)), vector((0,0,-1)))
            False

        ::

            sage: C = ChristoffelGraph((2,5))
            sage: C.has_edge(vector((0,0)),vector((1,0)))
            True
            sage: C.has_edge(vector((0,0)),vector((-1,0)))
            False
            sage: C.has_edge(vector((-1,1)),vector((1,0)))
            False
        """
        F = self.level_value
        sum_s_minus_p = sum(s-p)
        if sum_s_minus_p == 1:
            return F(p) < F(s)
        elif sum_s_minus_p == -1:
            return F(p) > F(s)
        else:
            return False
            #raise ValueError("vector p=(%s) and s(=%s) should difer by a canonical vector" % (p,s))

    def _repr_(self):
        r"""
        String representation.

        EXAMPLES::

            sage: from slabbe import ChristoffelGraph
            sage: ChristoffelGraph((2,5))
            Christoffel set of edges for normal vector v=(2, 5)
            sage: ChristoffelGraph((2,5,8))
            Christoffel set of edges for normal vector v=(2, 5, 8)
        """
        s = "Christoffel set of edges for normal vector v=%s" % self._v
        return s

    def kernel_vector(self, way='LLL', verbose=False):
        r"""
        todo: clean this

        EXAMPLES::

            sage: from slabbe import ChristoffelGraph
            sage: C = ChristoffelGraph((2,5,7))
            sage: C.kernel_vector()
            [(-1, -1, 1), (3, -4, 0)]

        """
        from sage.arith.misc import gcd
        if way == 'vect_gcd':
            a,b,c = self._v
            gcd_ac = gcd(a,c)
            gcd_bc = gcd(b,c)
            U = ua,ub,uc = vector((c,0,-a)) / gcd(a,c)
            V = va,vb,vc = vector((0,c,-b)) / gcd(b,c)
            rows = U,V
        elif way == 'echelon':
            a,b,c = self._v
            m = matrix(ZZ, 4, [1,1,1,c,0,-a,0,c,-b,b,-a,0])
            me = m.echelon_form()
            if verbose:
                print(me)
            rows = me[1],me[2]
        elif way == 'LLL':
            dim = self.dimension()
            if dim == 3:
                a,b,c = self._v
                M = matrix(ZZ, 4, [1,1,1,c,0,-a,0,c,-b,b,-a,0])
            elif dim == 4:
                a,b,c,d = self._v
                M = matrix(ZZ, 7, (1,1,1,1,b,-a,0,0,c,0,-a,0,0,c,-b,0,d,0,0,-a,0,d,0,-b,0,0,d,-c))
            else:
                raise ValueError("dimension (=%s) must be 3 or 4" % dim)
            rows = M.LLL().rows()
            VS = rows[0].parent()
            zero = VS(0)
            un = VS((1,)*dim)
            assert zero in rows, "(0,0,0) not in LLL result"
            assert un in rows, "(1,1,1) not in LLL result"
            while zero in rows: rows.remove(zero)
            while un in rows: rows.remove(un)
        elif way == 'vect':
            a,b,c = self._v
            U = ua,ub,uc = vector((c,0,-a))
            V = va,vb,vc = vector((0,c,-b))
            rows = U,V
        else:
            raise ValueError("unknown way")
        R = matrix(rows)
        if sum(map(abs, R.minors(dim-1))) != sum(map(abs,self._v)):
            print(R)
            print(R.minors(dim-1))
            print(sum(map(abs, R.minors(dim))))
            print(sum(map(abs,self._v)))
            raise Exception("The result (=%s) is false " % rows)
        return rows

    def tikz_kernel(self, projmat=M3to2, scale=1, edges=True,
            points=True, label=False, point_kwds={}, edge_kwds={}, extra_code='',
            way='LLL', kernel_vector=None):
        r"""
        INPUT:

        - ``projmat`` -- (default: M3to2) 2 x dim projection
          matrix where dim is the dimensoin of self, the isometric
          projection is used by default
        - ``scale`` -- real number (default: 1), scaling constant for the
          whole figure
        - ``edges`` - bool (optional, default: ``True``), whether to draw
          edges
        - ``points`` - bool (optional, default: ``True``), whether to draw
          points
        - ``point_kwds`` - dict (default: ``{}``)
        - ``edge_kwds`` - dict (default: ``{}``)
        - ``extra_code`` -- string (default: ``''``), extra tikz code to add
        - ``way`` -- string (default: ``'LLL'``), the way the base of the
          kernel is computed
        - ``kernel_vector`` -- list (default: ``None``), the vectors, if
          None it uses ``kernel_vector()`` output.

        EXAMPLES::

            sage: from slabbe import ChristoffelGraph
            sage: C = ChristoffelGraph((2,3,5))
            sage: tikz = C.tikz_kernel()
            sage: tikz
            \documentclass[tikz]{standalone}
            \usepackage{amsmath}
            \begin{document}
            \begin{tikzpicture}
            [scale=1]
            \clip (-5.239453693, 2.025) -- (-5.239453693, -1.025) -- (-4.330127019, -1.55) -- (0.0, -1.05) -- (0.909326674, -0.525) -- (0.909326674, 2.525) -- (0.0, 3.05) -- (-4.330127019, 2.55) -- cycle;
            \draw[very thick, blue] (0.00000, 0.00000) -- (-0.86603, -0.50000);
            \draw[very thick, blue] (0.00000, 0.00000) -- (0.86603, -0.50000);
            ...
            ... 296 lines not printed (24077 characters in total) ...
            ...
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (-6.92820, 5.00000) {};
            \filldraw[fill=white,very thick,dotted,opacity=0.5,even odd rule]
            (-5.239453693, 2.025) -- (-5.239453693, -1.025) -- (-4.330127019, -1.55) -- (0.0, -1.05) -- (0.909326674, -0.525) -- (0.909326674, 2.525) -- (0.0, 3.05) -- (-4.330127019, 2.55) -- cycle
            (-4.330127019, 1.5) -- (-4.330127019, -0.5) -- (0.0, 0.0) -- (0.0, 2.0) -- cycle;
            \end{tikzpicture}
            \end{document}
        """
        # kernel to clip
        if kernel_vector:
            K = kernel_vector
        else:
            K = self.kernel_vector(way=way)

        if self.dimension() == 4:
            projmat_tube = M4to3
        elif self.dimension() == 3:
            projmat_tube = M3to2
        else:
            raise NotImplementedError("dim must be 3 or 4")

        zero = self.an_element().parent(0)
        para_base_edges = [(zero, vector(U)) for U in K]
        para = [sum(p) for p in itertools.product(*para_base_edges)]
        base_edges = self.base_edges()
        hex = base_edges + [-v for v in base_edges]
        clip_ambiant_dim = [p+1.05*h for p in para for h in hex]
        clip_proj_dim = [projmat_tube * c for c in clip_ambiant_dim]

        Z = list(zip(*clip_proj_dim))
        minZ = vector(RR, map(min, Z)) - vector(RR, (2,)*len(Z))
        maxZ = vector(RR, map(max, Z)) + vector(RR, (2,)*len(Z))
        tube = DiscreteTube(*zip(minZ,maxZ), projmat=projmat_tube)
        plane = DiscretePlane(self._v, sum(self._v))
        I = self & tube & plane
        #I = Intersection3d(I)

        if self.dimension() == 4:
            projmat = M3to2*M4to3
        elif self.dimension() == 3:
            projmat = M3to2
        else:
            raise NotImplementedError("dim must be 3 or 4")
        tikz = I.tikz(projmat=projmat, scale=scale, edges=edges,
                points=points, point_kwds=point_kwds, edge_kwds=edge_kwds,
                clip=clip_ambiant_dim, contour=para, extra_code=extra_code)

        return tikz





# -*- coding: utf-8 -*-
r"""
The code contained in `arXiv:1906.01104`__

S. Labbé, Rauzy induction of polygon partitions and toral $\Z^2$-rotations
arXiv:1906.01104, June 2019, 36 p. (v2: revised in May 2020).

__ https://arxiv.org/abs/1906.01104

EXAMPLES::

    sage: from slabbe.arXiv_1906_01104 import beta8,beta9,tau
    sage: beta8*beta9*tau
    Substitution 2d: {0: [[17]], 1: [[12]], 2: [[16, 10]], 3: [[16, 9]], 
    4: [[17, 7]], 5: [[12, 7]], 6: [[16], [2]], 7: [[14], [4]], 
    8: [[17], [2]], 9: [[13], [3]], 10: [[13], [2]], 11: [[12], [2]], 
    12: [[15, 11], [5, 1]], 13: [[18, 10], [4, 1]], 14: [[16, 10], [3, 1]],
    15: [[16, 9], [2, 0]], 16: [[14, 6], [4, 1]], 17: [[14, 8], [4, 1]], 
    18: [[13, 6], [3, 1]]}

"""
#*****************************************************************************
#       Copyright (C) 2019-2020 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from sage.rings.polynomial.polynomial_ring import polygen
from sage.rings.rational_field import QQ
from sage.rings.number_field.number_field import NumberField
from sage.modules.free_module_element import vector
from sage.matrix.constructor import matrix
from sage.geometry.polyhedron.library import polytopes

# Polyhedron exchange transformations and induction of polyhedron partitions are
# implemented in Sage with the optional package slabbe
from slabbe import PolyhedronExchangeTransformation as PET
from slabbe import Substitution2d

# We construct the golden mean as a element of a quadratic number field
# because it is more efficient for arithmetic operations and comparisons:
z = polygen(QQ, 'z')
K = NumberField(z**2-z-1, 'phi', embedding=QQ(1.6))
phi = K.gen()

# The initial torus and Z^2-actions
Gamma0 = matrix.column([(phi,0), (1,phi+3)])
fundamental_domain = polytopes.parallelotope([(phi,0), (0,phi+3)])
R0e1 = PET.toral_translation(Gamma0, vector((1,0)), fundamental_domain)
R0e2 = PET.toral_translation(Gamma0, vector((0,1)), fundamental_domain)

# The initial partition P0
from slabbe.arXiv_1903_06137 import jeandel_rao_wang_shift_partition
P0 = jeandel_rao_wang_shift_partition()

# Compute P1 and beta0
y_le_1 = [1, 0, -1]   # syntax for the inequality y <= 1
P1,beta0 = R0e2.induced_partition(y_le_1, P0, substitution_type='column')
R1e1,_ = R0e1.induced_transformation(y_le_1)
R1e2,_ = R0e2.induced_transformation(y_le_1)

# The lattices (we don't need them, it's just to show how to construct them)
Gamma2 = Gamma1 = matrix.column([(phi,0), (0,1)])

# Change of base (shearing the action)
P2 = P1
R2e1 = R1e1
R2e2 = (R1e1 * R1e2).merge_atoms_with_same_translation()

# Compute P3 and beta2
x_le_1 = [1, -1, 0]   # syntax for x <= 1
P3,beta2 = R2e1.induced_partition(x_le_1, P2, substitution_type='row')
R3e1,_ = R2e1.induced_transformation(x_le_1)
R3e2,_ = R2e2.induced_transformation(x_le_1)

# Compute P4 and beta3
x_le_phi_inv = [phi**-1, -1, 0]    # syntax for x <= phi^-1
P4,beta3 = R3e1.induced_partition(x_le_phi_inv, P3, substitution_type='row')
R4e1,_ = R3e1.induced_transformation(x_le_phi_inv)
R4e2,_ = R3e2.induced_transformation(x_le_phi_inv)

# Compute P5 and beta4
y_le_phi_inv = [phi**-1, 0, -1]    # syntax for y <= phi^-1
P5,beta4 = R4e2.induced_partition(y_le_phi_inv, P4, substitution_type='column')
R5e1,_ = R4e1.induced_transformation(y_le_phi_inv)
R5e2,_ = R4e2.induced_transformation(y_le_phi_inv)

# Renormalization
P5_scaled = (-phi*P5).translate((1,1))
R5e1_scaled = (-phi*R5e1).translate_domain((1,1))
R5e2_scaled = (-phi*R5e2).translate_domain((1,1))

# Compute P6 and beta5
P6,beta5 = R5e1_scaled.induced_partition(x_le_phi_inv, P5_scaled, substitution_type='row')
R6e1,_ = R5e1_scaled.induced_transformation(x_le_phi_inv)
R6e2,_ = R5e2_scaled.induced_transformation(x_le_phi_inv)

# Compute P7 and beta6
P7,beta6 = R6e2.induced_partition(y_le_phi_inv, P6, substitution_type='column')
R7e1,_ = R6e1.induced_transformation(y_le_phi_inv)
R7e2,_ = R6e2.induced_transformation(y_le_phi_inv)

# Renormalization
P7_scaled = (-phi*P7).translate((1,1))
R7e1_scaled = (-phi*R7e1).translate_domain((1,1))
R7e2_scaled = (-phi*R7e2).translate_domain((1,1))

# Compute P8 and beta7
P8,beta7 = R7e1_scaled.induced_partition(x_le_phi_inv, P7_scaled, substitution_type='row')
R8e1,_ = R7e1_scaled.induced_transformation(x_le_phi_inv)
R8e2,_ = R7e2_scaled.induced_transformation(x_le_phi_inv)

# Compute P9 and beta8
P9,beta8 = R8e2.induced_partition(y_le_phi_inv, P8, substitution_type='column')
R9e1,_ = R8e1.induced_transformation(y_le_phi_inv)
R9e2,_ = R8e2.induced_transformation(y_le_phi_inv)

# Renormalization
P9_scaled = (-phi*P9).translate((1,1))
R9e1_scaled = (-phi*R9e1).translate_domain((1,1))
R9e2_scaled = (-phi*R9e2).translate_domain((1,1))

# Compute P10 and beta9
P10,beta9 = R9e1_scaled.induced_partition(x_le_phi_inv, P9_scaled, substitution_type='row')
R10e1,_ = R9e1_scaled.induced_transformation(x_le_phi_inv)
R10e2,_ = R9e2_scaled.induced_transformation(x_le_phi_inv)

# the self-similarity for P8
assert P8 == P10
tau = Substitution2d.from_permutation(P8.keys_permutation(P10))
self_similarity_P8 = beta8*beta9*tau

# We may check that the self-similarity for $\Pcal_8$ satisfies 
# $\zeta^{-1}\beta_8\beta_9\tau\zeta=\beta_{\mathcal{U}}$.

zeta = Substitution2d.from_permutation({0:0, 1:1, 2:9, 3:7, 4:8, 5:11, 6:10, 
7:6, 8:2, 9:4, 10:5, 11:3, 12:18, 13:14, 14:16, 15:13, 16:12, 17:17, 18:15})

betaU = Substitution2d({0: [[17]], 1: [[16]], 2: [[15], [11]], 3: [[13], [9]], 4: [[17], [8]], 5: [[16], [8]], 6: [[15], [8]], 7: [[14], [8]], 8: [[14, 6]], 9: [[17, 3]], 10: [[16, 3]], 11: [[14, 2]], 12: [[15, 7], [11, 1]], 13: [[14, 6], [11, 1]], 14: [[13, 7], [9, 1]], 15: [[12, 6], [9, 1]], 16: [[18, 5], [10, 1]], 17: [[13, 4], [9, 1]], 18: [[14, 2], [8, 0]]})

self_sim_PU_deduced = zeta.inverse()*beta8*beta9*tau*zeta
assert betaU == self_sim_PU_deduced 

